#include <pic.h>

#ifdef _16F676
	#define XTAL_4MHZ
	__CONFIG(INTOSCIO & WDTDIS & UNPROTECT & BOREN & MCLRDIS)
#elif _16F688
	#define XTAL_8MHZ
	__CONFIG(INTIO & WDTDIS & UNPROTECT & BOREN & MCLRDIS)
#error Not yet supported !
#endif

//--- PORT DEFINITIONS ---------------------------------------------------
//               76543210
#define  TrisA 0b00001000                
//               ||||||||
//               |||||||FR_LED
//               ||||||OV_LED
//               |||||BUF_OV_LED
//               ||||MODE
//               |||A0 
//               ||E
//               |---
//               ---

#define  TrisC 0b00000000
//               76543210
//               ||||||||
//               |||||||D4
//               ||||||D5
//               |||||D6
//               ||||D7
//               |||TX
//               ||RX
//               |---
//               ---

#define LCD_SCLK               RA0 // PIC-13
#define LCD_SDA                RA1 // PIC-12
#define LCD_DC                 RA2 // PIC-11
#define LCD_CE                 RC0 // PIC-10
#define LCD_RST                RC1 // PIC-9

#define LED 		       RC2 // PIC-8

#include "types.h"

#include "delay.c"

#define LCD_PAGE_HANDLER LCDPageHandler();

void LCDPageHandler(void) {
  Delay10ms (45);
}

#include "nokia3410.c"

void main(void) {
  TRISA = 0;
  TRISC = 0;

  OPTION = 0b01000010; // pull-up, int/RA2 on 0->1; Prescalder 8 to TMR0;

#ifdef _16F688
  //         76453210
  OSCCON = 0b01110001; // 8 MHz clock;
#endif

  PORTA = 0; 
  PORTC = 0;

#if defined(_16F688)
  CMCON0 = 0b00000111; // comparator is turned off;
  WDTCON = 0b10001; // 1000 = 1:8192 (1:16 relative to default)
#elif defined(_16F676)
  CMCON = 0b00000111; // comparator is turned off;
#endif

  ANSEL = 0;

  ADCON1 = 0b01010000;      //  Select the Clock as Fosc/16

  ADIE = 0;

  ADCON0 = 0b10000000;     //  Turn off the ADC
                              //   Bit 7 - Right Justified Sample
                              //   Bit 6 - Dont Use VRef
                              //   Bit 4:2 - Channel 0
                              //   Bit 1 - Do not Start
                              //   Bit 0 - Turn on ADC  

  Delay10ms (10);    
  while (1) { 
    LcdInit ();
    LcdStr ("Privet!");
    CLRWDT();
    LED = 1;
    Delay10ms (10);    
    LED = 0;
    Delay10ms (77);    
  }
}
